from mailtrail.backends.base import MailTrailBase


class EmailBackend(MailTrailBase):
    """
    The database backend is very useful for debugging templates and
    maybe for creating internal messaging systems - that's up to you
    to figure out though!
    """
    BACKEND = 'DATABASE'

    def __init__(self, *args, **kwargs):
        pass

    def send_messages(self, email_messages):
        for message in email_messages:
            self.save_message(message)



from django.contrib import admin
from django.conf import settings
from django.conf.urls import url
from mailtrail.models import Email, get_recipient_model, get_recipient_model_attribute
from mailtrail import views


@admin.register(Email)
class EmailAdmin(admin.ModelAdmin):
    change_form_template = 'email/change.html'

    list_display = ['subject', 'recipient_list', 'created']
    list_filter = ['recipients__{}'.format(get_recipient_model_attribute()), 'created', 'backend']
    exclude = ['payload']
    readonly_fields = ['plaintext_message', 'html_message', 'created', 'from_email', 'subject', 'backend', 'recipients', 'is_forwarded']

    def change_view(self, request, object_id, **kwargs):
        kwargs['extra_context'] = {
            'pk': object_id
        }
        return super(EmailAdmin, self).change_view(request, object_id, **kwargs)

    def get_urls(self):
        urls = super(EmailAdmin, self).get_urls()
        my_urls = [
            url(r'^(?P<pk>[0-9a-f-]+)/resend/$', views.EmailResendView.as_view(), name="mailtrail_email_resend"),
            url(r'^(?P<pk>[0-9a-f-]+)/raw/$', views.EmailRawView.as_view(), name="mailtrail_email_raw"),
            url(r'^(?P<pk>[0-9a-f-]+)/forward/$', views.EmailForwardView.as_view(), name="mailtrail_email_forward")
        ]
        return my_urls + urls


class RecipientAdmin(admin.ModelAdmin):
    exclude = []
    readonly_fields = [get_recipient_model_attribute()]


def get_recipient_model_admin():
    if hasattr(settings, 'MAILTRAIL_RECIPIENT_MODEL_ADMIN'):
        return getattr(settings, 'MAILTRAIL_RECIPIENT_MODEL_ADMIN')
    else:
        return RecipientAdmin

admin.site.register(get_recipient_model(), get_recipient_model_admin())



from mailtrail.models import Email, get_recipient_model, get_recipient_model_attribute


class MailTrailBase:
    BLANK = ''
    BACKEND = None

    def _get_content(self, msg, content_type, fallback=None):
        payload = fallback

        for part in msg.walk():
            if part.get_content_type() == content_type:
                payload = part.get_payload()

        return payload

    def save_message(self, message):
        """
        Save a sent message in the database
        """
        Recipient = get_recipient_model()

        # Uses email library to parse messages
        payload = message.message()

        plaintext = self._get_content(payload, 'text/plain', fallback='')
        html = self._get_content(payload, 'text/html')

        email = Email.objects.create(
            subject=message.subject,
            payload=message.message(),
            plaintext_message=plaintext,
            html_message=html,
            from_email=message.from_email,
            backend=self.BACKEND
        )

        for recipient_email in message.recipients():
            # Create recipients if they do not exist
            data = {
                get_recipient_model_attribute(): recipient_email
            }
            recipient = Recipient.objects.get_or_create(**data)[0]
            email.recipients.add(recipient)



from django.db import models
from django.apps import apps
from django.conf import settings
from django.utils import timezone
import uuid


class Recipient(models.Model):
    """
    This recipient model can be extended if you would like to link emails to
    user models. However, the `email` field must always be present and all
    other fields should be nullable.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email = models.EmailField(unique=True)

    def __str__(self):
        return self.email


def get_recipient_model(as_instance=True):
    if not hasattr(settings, 'MAILTRAIL_RECIPIENT_MODEL'):
        return Recipient
    else:
        model = getattr(settings, 'MAILTRAIL_RECIPIENT_MODEL')
        if as_instance:
            return apps.get_model(model)
        else:
            return model


def get_recipient_model_attribute():
    return getattr(settings, 'MAILTRAIL_RECIPIENT_MODEL_ATTRIBUTE', 'email')



class Email(models.Model):
    """
    Email model for catching and storing all sent emails via `django.core.mail`.
    """
    RECIPIENT_MODEL = get_recipient_model(as_instance=False)

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    subject = models.CharField(max_length=255, blank=True)
    payload = models.TextField(blank=True)
    plaintext_message = models.TextField(blank=True)
    html_message = models.TextField(blank=True, null=True)
    from_email = models.EmailField()
    recipients = models.ManyToManyField(RECIPIENT_MODEL, related_name='emails')

    is_forwarded = models.BooleanField(default=False)

    # An identifier to track what backend was used
    backend = models.CharField(max_length=32, null=True, blank=True)

    # When the email was first sent
    created = models.DateTimeField(default=timezone.now)

    def total_recipients(self):
        return self.recipients.all().count()

    def recipient_list(self):
        total_recipients = self.total_recipients()
        recipient_emails = ', '.join(getattr(recipient, get_recipient_model_attribute()) for recipient in self.recipients.all()[:3])

        if total_recipients > 3:
            recipient_emails += ' and {} more...'.format(total_recipients - 3)

        return recipient_emails

    def __str__(self):
        return '[{subject}] {recipients}'.format(
            subject=self.subject,
            recipients=self.recipient_list()
        )

    class Meta:
        ordering = ['subject', 'created']



# # # # # # # # # # # # #
#   ** TEST MODELS **   #
# # # # # # # # # # # # #

class DummyRecipient(models.Model):
    """
    Dummy recipient model for testing custom recipient model. Please
    don't reference this in production.
    """
    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    email_address = models.EmailField()
    name = models.CharField(max_length=50)

    def __str__(self):
        return '{} - {}'.format(self.email_address, self.name)



from mailtrail.backends.base import MailTrailBase
from django.core.mail.backends.console import EmailBackend as ConsoleEmailBackend


class EmailBackend(MailTrailBase, ConsoleEmailBackend):
    BACKEND = 'CONSOLE'

    def send_messages(self, email_messages):
        for message in email_messages:
            self.save_message(message)

        return super(EmailBackend, self).send_messages(email_messages)



from django.test import TestCase
from django.test.utils import override_settings
from django.core import mail
from django.apps import apps
from django.conf import settings
from backends import console, database, filebased
from models import Email, Recipient
from mock import patch
from django.urls import reverse
import os, shutil


class ModelTestCase(TestCase):
    def test_create_email(self):
        recipient = Recipient.objects.create(
            email='me@me.com'
        )

        email = Email.objects.create(
            subject='Test subject',
            plaintext_message='Test message',
            from_email='test@testuser.com'
        )
        email.recipients.add(recipient)

        self.assertEqual(Recipient.objects.all().count(), 1)
        self.assertEqual(Email.objects.all().count(), 1)
        self.assertEqual(email.recipients.all().count(), 1)

    def test_create_email_multiple_recipients(self):
        email = Email.objects.create(
            subject='Test subject',
            plaintext_message='Test message',
            from_email='test@testuser.com'
        )

        recipients = [
            'testuser1@test.com',
            'testuser2@test.com',
            'testuser3@test.com'
        ]

        for recipient_email in recipients:
            recipient = Recipient.objects.create(
                email=recipient_email
            )
            email.recipients.add(recipient)

        self.assertEqual(Recipient.objects.all().count(), 3)

        for recipient_email in recipients:
            self.assertTrue(Email.objects.filter(recipients__email=recipient_email).exists())

    def test_create_recipient(self):
        Recipient.objects.create(
            email='me@me.com'
        )

        self.assertEqual(Recipient.objects.all().count(), 1)

    def test_recipient_count(self):
        recipient = Recipient.objects.create(
            email='me@me.com'
        )

        email = Email.objects.create(
            subject='Test subject',
            plaintext_message='Test message',
            from_email='test@testuser.com'
        )
        email.recipients.add(recipient)

        self.assertEqual(email.total_recipients(), 1)

    @override_settings(MAILTRAIL_RECIPIENT_MODEL='mailtrail.DummyRecipient',
                       MAILTRAIL_RECIPIENT_MODEL_ATTRIBUTE='email_address')
    def test_custom_recipient_model(self):
        Model = apps.get_model(settings.MAILTRAIL_RECIPIENT_MODEL)
        data = {
            settings.MAILTRAIL_RECIPIENT_MODEL_ATTRIBUTE: 'test@testuser.com',
            'name': 'Mr. T. Esting'
        }
        recipient = Model.objects.create(**data)

        self.assertEqual(recipient.email_address, 'test@testuser.com')
        self.assertEqual(recipient.name, 'Mr. T. Esting')
        self.assertEqual(Model.objects.all().count(), 1)


class BackendTestCase(TestCase):
    FILE_TEST_PATH = '/tmp/mailtrail-tests'

    @patch('django.core.mail.backends.locmem.EmailBackend', console.EmailBackend)
    def test_send_email_console(self):
        mail.send_mail('Test subject', 'Test message', 'test@testuser.com', ('test@testuser.com',), html_message='<p>html</p>')

        all_emails = Email.objects.all()
        self.assertEquals(all_emails.count(), 1)

        email = all_emails.first()
        self.assertEquals(email.subject, 'Test subject')
        self.assertEquals(email.html_message, '<p>html</p>')
        self.assertEquals(email.from_email, 'test@testuser.com')
        self.assertEquals(Recipient.objects.all().count(), 1)

    @patch('django.core.mail.backends.locmem.EmailBackend', console.EmailBackend)
    def test_send_email_duplicate_recipients(self):
        mail.send_mail('Test subject', 'Test message', 'test@testuser.com',
                       ('test@testuser.com', 'test@testuser.com', 'test2@testuser.com'),
                       html_message='<p>html</p>')

        self.assertEquals(Email.objects.all().count(), 1)

        email = Email.objects.all().first()
        self.assertEqual(email.recipients.all().count(), 2)

        self.assertEqual(Recipient.objects.filter(email='test@testuser.com').count(), 1)
        self.assertEqual(Recipient.objects.filter(email='test2@testuser.com').count(), 1)

    @patch('django.core.mail.backends.locmem.EmailBackend', database.EmailBackend)
    def test_send_email_database(self):
        mail.send_mail('Test subject', 'Test message', 'test@testuser.com', ('test@testuser.com',),
                       html_message='<p>html</p>')

        self.assertEquals(Email.objects.all().count(), 1)

        email = Email.objects.all().first()
        self.assertEqual(email.recipients.all().count(), 1)

        self.assertEqual(Recipient.objects.filter(email='test@testuser.com').count(), 1)

    @patch('django.core.mail.backends.locmem.EmailBackend', filebased.EmailBackend)
    @override_settings(EMAIL_FILE_PATH=FILE_TEST_PATH)
    def test_send_email_database(self):
        mail.send_mail('Test subject', 'Test message', 'test@testuser.com', ('test@testuser.com',),
                       html_message='<p>html</p>')

        self.assertEquals(Email.objects.all().count(), 1)

        email = Email.objects.all().first()
        self.assertEqual(email.recipients.all().count(), 1)
        self.assertEqual(Recipient.objects.filter(email='test@testuser.com').count(), 1)

        num_files = len([f for f in os.listdir(self.FILE_TEST_PATH)])
        self.assertEqual(num_files, 1)

    @patch('django.core.mail.backends.locmem.EmailBackend', database.EmailBackend)
    def test_resend_email(self):
        mail.send_mail('Test subject', 'Test message', 'test@testuser.com',
                       ('test@testuser.com', 'test@testuser.com', 'test2@testuser.com'),
                       html_message='<p>html</p>')

        self.assertEqual(Email.objects.all().count(), 1)

        email = Email.objects.all().first()
        url = reverse('admin:mailtrail_email_resend', kwargs={'pk': email.pk})
        response = self.client.post(url)

        self.assertEqual(response.status_code, 200)
        self.assertEqual(Email.objects.all().count(), 2)

    @patch('django.core.mail.backends.locmem.EmailBackend', database.EmailBackend)
    def test_send_empty_email(self):
        mail.send_mail('Test subject', '', 'test@testuser.com',
                       ('test@testuser.com', 'test@testuser.com', 'test2@testuser.com'),
                       html_message='')

        self.assertEqual(Email.objects.all().count(), 1)

        mail.send_mail('Test subject', 'empty html', 'test@testuser.com',
                       ('test@testuser.com', 'test@testuser.com', 'test2@testuser.com'),
                       html_message='')

        self.assertEqual(Email.objects.all().count(), 2)

        mail.send_mail('Test subject', '', 'test@testuser.com',
                       ('test@testuser.com', 'test@testuser.com', 'test2@testuser.com'),
                       html_message='empty plaintext')

        self.assertEqual(Email.objects.all().count(), 3)

    def tearDown(self):
        if os.path.exists(self.FILE_TEST_PATH):
            shutil.rmtree(self.FILE_TEST_PATH)

        super(BackendTestCase, self).tearDown()



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-20 13:13
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='DummyRecipient',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email_address', models.EmailField(max_length=254)),
                ('name', models.CharField(max_length=50)),
            ],
        ),
        migrations.CreateModel(
            name='Email',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('subject', models.CharField(blank=True, max_length=255)),
                ('payload', models.TextField(blank=True)),
                ('plaintext_message', models.TextField(blank=True)),
                ('html_message', models.TextField(blank=True, null=True)),
                ('from_email', models.EmailField(max_length=254)),
                ('is_forwarded', models.BooleanField(default=False)),
                ('backend', models.CharField(blank=True, max_length=32, null=True)),
                ('created', models.DateTimeField(default=django.utils.timezone.now)),
            ],
            options={
                'ordering': ['subject', 'created'],
            },
        ),
        migrations.CreateModel(
            name='Recipient',
            fields=[
                ('id', models.UUIDField(default=uuid.uuid4, editable=False, primary_key=True, serialize=False)),
                ('email', models.EmailField(max_length=254, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name='email',
            name='recipients',
            field=models.ManyToManyField(related_name='emails', to='mailtrail.Recipient'),
        ),
    ]



from mailtrail.backends.base import MailTrailBase
from django.core.mail.backends.smtp import EmailBackend as SMTPEmailBackend


class EmailBackend(MailTrailBase, SMTPEmailBackend):
    BACKEND = 'SMTP'

    def send_messages(self, email_messages):
        for message in email_messages:
            self.save_message(message)

        return super(EmailBackend, self).send_messages(email_messages)



from django.views.generic import View
from django.template.response import TemplateResponse
from django.core import urlresolvers
from django.shortcuts import get_object_or_404, HttpResponseRedirect
from django.core import mail
from mailtrail.models import Email, get_recipient_model_attribute


class EmailResendView(View):
    def get(self, request, pk):
        email = get_object_or_404(Email, pk=pk)
        recipients = email.recipients.all()

        context = {
            'email': email,
            'recipients': {
                'list': [getattr(recipient, get_recipient_model_attribute()) for recipient in recipients],
                'total': recipients.count()
            }
        }

        return TemplateResponse(request, template='email/resend.html', context=context)

    def post(self, request, pk):
        email = get_object_or_404(Email, pk=pk)
        recipients = email.recipients.all()

        total_recipients = recipients.count()

        context = {
            'email': email,
            'recipients': {
                'list': [getattr(recipient, get_recipient_model_attribute()) for recipient in recipients],
                'total': total_recipients
            },
            'messages': [
                'Successfully resent email to {} recipient{}'.format(total_recipients, 's' if total_recipients != 1 else '')
            ]
        }

        mail.send_mail(email.subject, email.plaintext_message, email.from_email,
                       email.recipients.all(), html_message=email.html_message)

        return TemplateResponse(request, template='email/resend.html', context=context)


class EmailRawView(View):
    def get(self, request, pk):
        email = get_object_or_404(Email, pk=pk)

        context = {
            'email': email,
            'bytes': len(email.payload)
        }

        return TemplateResponse(request, template='email/raw.html', context=context)


class EmailForwardView(View):
    def get(self, request, pk):
        email = get_object_or_404(Email, pk=pk)

        context = {
            'email': email
        }

        return TemplateResponse(request, template='email/forward.html', context=context)

    def post(self, request, pk):
        email = get_object_or_404(Email, pk=pk)

        data = request.POST

        recipients = data['recipients_new'].replace(' ', '').split(',')
        if 'include_recipients' in data.keys():
            [recipients.append(recipient.email) for recipient in email.recipients.all()]

        mail.send_mail(email.subject, email.plaintext_message, email.from_email,
                       recipients, html_message=email.html_message)

        forwarded_email = Email.objects.all().order_by('created').first()
        forwarded_email.is_forwarded = True
        forwarded_email.save()

        context = {
            'email': email
        }

        return HttpResponseRedirect(urlresolvers.reverse('admin:mailtrail_email_change', args=(forwarded_email.pk,)))
